@@ -21,7 +21,6 @@ import logging
 from typing import TYPE_CHECKING, Optional
 
 from synapse.api.errors import Codes, StoreError, SynapseError
-from synapse.handlers.device import DeviceHandler
 from synapse.types import Requester
 
 if TYPE_CHECKING:
@@ -36,17 +35,7 @@ class SetPasswordHandler:
     def __init__(self, hs: "HomeServer"):
         self.store = hs.get_datastores().main
         self._auth_handler = hs.get_auth_handler()
-
-        # We don't need the device handler if password changing is disabled.
-        # This allows us to instantiate the SetPasswordHandler on the workers
-        # that have admin APIs for MAS
-        if self._auth_handler.can_change_password():
-            # This can only be instantiated on the main process.
-            device_handler = hs.get_device_handler()
-            assert isinstance(device_handler, DeviceHandler)
-            self._device_handler: Optional[DeviceHandler] = device_handler
-        else:
-            self._device_handler = None
+        self._device_handler = hs.get_device_handler()
 
     async def set_password(
         self,
@@ -58,9 +47,6 @@ class SetPasswordHandler:
         if not self._auth_handler.can_change_password():
             raise SynapseError(403, "Password change disabled", errcode=Codes.FORBIDDEN)
 
-        # We should have this available only if password changing is enabled.
-        assert self._device_handler is not None
-
         try:
             await self.store.user_set_password_hash(user_id, password_hash)
         except StoreError as e:
