@@ -38,6 +38,7 @@ from synapse.http.servlet import parse_json_object_from_request
 from synapse.http.site import SynapseRequest
 from synapse.logging import opentracing
 from synapse.logging.opentracing import trace_with_opname
+from synapse.metrics import SERVER_NAME_LABEL
 from synapse.types import JsonDict
 from synapse.util.caches.response_cache import ResponseCache
 from synapse.util.cancellation import is_function_cancellable
@@ -51,13 +52,13 @@ logger = logging.getLogger(__name__)
 _pending_outgoing_requests = Gauge(
     "synapse_pending_outgoing_replication_requests",
     "Number of active outgoing replication requests, by replication method name",
-    ["name"],
+    labelnames=["name", SERVER_NAME_LABEL],
 )
 
 _outgoing_request_counter = Counter(
     "synapse_outgoing_replication_requests",
     "Number of outgoing replication requests, by replication method name and result",
-    ["name", "code"],
+    labelnames=["name", "code", SERVER_NAME_LABEL],
 )
 
 
@@ -121,16 +122,21 @@ class ReplicationEndpoint(metaclass=abc.ABCMeta):
     WAIT_FOR_STREAMS: ClassVar[bool] = True
 
     def __init__(self, hs: "HomeServer"):
+        self.server_name = hs.hostname
+
         if self.CACHE:
             self.response_cache: ResponseCache[str] = ResponseCache(
-                hs.get_clock(), "repl." + self.NAME, timeout_ms=30 * 60 * 1000
+                clock=hs.get_clock(),
+                name="repl." + self.NAME,
+                server_name=self.server_name,
+                timeout_ms=30 * 60 * 1000,
             )
 
         # We reserve `instance_name` as a parameter to sending requests, so we
         # assert here that sub classes don't try and use the name.
-        assert (
-            "instance_name" not in self.PATH_ARGS
-        ), "`instance_name` is a reserved parameter name"
+        assert "instance_name" not in self.PATH_ARGS, (
+            "`instance_name` is a reserved parameter name"
+        )
         assert (
             "instance_name"
             not in signature(self.__class__._serialize_payload).parameters
@@ -200,13 +206,17 @@ class ReplicationEndpoint(metaclass=abc.ABCMeta):
         parameter to specify which instance to hit (the instance must be in
         the `instance_map` config).
         """
+        server_name = hs.hostname
         clock = hs.get_clock()
         client = hs.get_replication_client()
         local_instance_name = hs.get_instance_name()
 
         instance_map = hs.config.worker.instance_map
 
-        outgoing_gauge = _pending_outgoing_requests.labels(cls.NAME)
+        outgoing_gauge = _pending_outgoing_requests.labels(
+            name=cls.NAME,
+            **{SERVER_NAME_LABEL: server_name},
+        )
 
         replication_secret = None
         if hs.config.worker.worker_replication_secret:
@@ -259,9 +269,9 @@ class ReplicationEndpoint(metaclass=abc.ABCMeta):
                     url_args.append(txn_id)
 
                 if cls.METHOD == "POST":
-                    request_func: Callable[
-                        ..., Awaitable[Any]
-                    ] = client.post_json_get_json
+                    request_func: Callable[..., Awaitable[Any]] = (
+                        client.post_json_get_json
+                    )
                 elif cls.METHOD == "PUT":
                     request_func = client.put_json
                 elif cls.METHOD == "GET":
@@ -328,15 +338,27 @@ class ReplicationEndpoint(metaclass=abc.ABCMeta):
                     # We convert to SynapseError as we know that it was a SynapseError
                     # on the main process that we should send to the client. (And
                     # importantly, not stack traces everywhere)
-                    _outgoing_request_counter.labels(cls.NAME, e.code).inc()
+                    _outgoing_request_counter.labels(
+                        name=cls.NAME,
+                        code=e.code,
+                        **{SERVER_NAME_LABEL: server_name},
+                    ).inc()
                     raise e.to_synapse_error()
                 except Exception as e:
-                    _outgoing_request_counter.labels(cls.NAME, "ERR").inc()
+                    _outgoing_request_counter.labels(
+                        name=cls.NAME,
+                        code="ERR",
+                        **{SERVER_NAME_LABEL: server_name},
+                    ).inc()
                     raise SynapseError(
                         502, f"Failed to talk to {instance_name} process"
                     ) from e
 
-                _outgoing_request_counter.labels(cls.NAME, 200).inc()
+                _outgoing_request_counter.labels(
+                    name=cls.NAME,
+                    code=200,
+                    **{SERVER_NAME_LABEL: server_name},
+                ).inc()
 
                 # Wait on any streams that the remote may have written to.
                 for stream_name, position in result.pop(
