@@ -34,7 +34,9 @@ from typing import (
     Generic,
     Iterable,
     List,
+    Literal,
     Optional,
+    Set,
     Tuple,
     Type,
     TypeVar,
@@ -43,21 +45,22 @@ from typing import (
     overload,
 )
 
-from typing_extensions import Literal
-
-from twisted.internet import reactor
-from twisted.internet.interfaces import IReactorTime
+from twisted.internet import defer, reactor
 
 from synapse.config import cache as cache_config
-from synapse.metrics.background_process_metrics import wrap_as_background_process
+from synapse.metrics.background_process_metrics import (
+    run_as_background_process,
+)
 from synapse.metrics.jemalloc import get_jemalloc_stats
-from synapse.util import Clock, caches
+from synapse.types import ISynapseThreadlessReactor
+from synapse.util import caches
 from synapse.util.caches import CacheMetric, EvictionReason, register_cache
 from synapse.util.caches.treecache import (
     TreeCache,
     iterate_tree_cache_entry,
     iterate_tree_cache_items,
 )
+from synapse.util.clock import Clock
 from synapse.util.linked_list import ListNode
 
 if TYPE_CHECKING:
@@ -118,103 +121,121 @@ USE_GLOBAL_LIST = False
 GLOBAL_ROOT = ListNode["_Node"].create_root_node()
 
 
-@wrap_as_background_process("LruCache._expire_old_entries")
-async def _expire_old_entries(
-    clock: Clock, expiry_seconds: float, autotune_config: Optional[dict]
-) -> None:
+def _expire_old_entries(
+    server_name: str,
+    clock: Clock,
+    expiry_seconds: float,
+    autotune_config: Optional[dict],
+) -> "defer.Deferred[None]":
     """Walks the global cache list to find cache entries that haven't been
     accessed in the given number of seconds, or if a given memory threshold has been breached.
     """
-    if autotune_config:
-        max_cache_memory_usage = autotune_config["max_cache_memory_usage"]
-        target_cache_memory_usage = autotune_config["target_cache_memory_usage"]
-        min_cache_ttl = autotune_config["min_cache_ttl"] / 1000
 
-    now = int(clock.time())
-    node = GLOBAL_ROOT.prev_node
-    assert node is not None
+    async def _internal_expire_old_entries(
+        clock: Clock, expiry_seconds: float, autotune_config: Optional[dict]
+    ) -> None:
+        if autotune_config:
+            max_cache_memory_usage = autotune_config["max_cache_memory_usage"]
+            target_cache_memory_usage = autotune_config["target_cache_memory_usage"]
+            min_cache_ttl = autotune_config["min_cache_ttl"] / 1000
 
-    i = 0
+        now = int(clock.time())
+        node = GLOBAL_ROOT.prev_node
+        assert node is not None
 
-    logger.debug("Searching for stale caches")
+        i = 0
 
-    evicting_due_to_memory = False
+        logger.debug("Searching for stale caches")
 
-    # determine if we're evicting due to memory
-    jemalloc_interface = get_jemalloc_stats()
-    if jemalloc_interface and autotune_config:
-        try:
-            jemalloc_interface.refresh_stats()
-            mem_usage = jemalloc_interface.get_stat("allocated")
-            if mem_usage > max_cache_memory_usage:
-                logger.info("Begin memory-based cache eviction.")
-                evicting_due_to_memory = True
-        except Exception:
-            logger.warning(
-                "Unable to read allocated memory, skipping memory-based cache eviction."
-            )
+        evicting_due_to_memory = False
 
-    while node is not GLOBAL_ROOT:
-        # Only the root node isn't a `_TimedListNode`.
-        assert isinstance(node, _TimedListNode)
-
-        # if node has not aged past expiry_seconds and we are not evicting due to memory usage, there's
-        # nothing to do here
-        if (
-            node.last_access_ts_secs > now - expiry_seconds
-            and not evicting_due_to_memory
-        ):
-            break
-
-        # if entry is newer than min_cache_entry_ttl then do not evict and don't evict anything newer
-        if evicting_due_to_memory and now - node.last_access_ts_secs < min_cache_ttl:
-            break
-
-        cache_entry = node.get_cache_entry()
-        next_node = node.prev_node
-
-        # The node should always have a reference to a cache entry and a valid
-        # `prev_node`, as we only drop them when we remove the node from the
-        # list.
-        assert next_node is not None
-        assert cache_entry is not None
-        cache_entry.drop_from_cache()
-
-        # Check mem allocation periodically if we are evicting a bunch of caches
-        if jemalloc_interface and evicting_due_to_memory and (i + 1) % 100 == 0:
+        # determine if we're evicting due to memory
+        jemalloc_interface = get_jemalloc_stats()
+        if jemalloc_interface and autotune_config:
             try:
                 jemalloc_interface.refresh_stats()
                 mem_usage = jemalloc_interface.get_stat("allocated")
-                if mem_usage < target_cache_memory_usage:
-                    evicting_due_to_memory = False
-                    logger.info("Stop memory-based cache eviction.")
+                if mem_usage > max_cache_memory_usage:
+                    logger.info("Begin memory-based cache eviction.")
+                    evicting_due_to_memory = True
             except Exception:
                 logger.warning(
-                    "Unable to read allocated memory, this may affect memory-based cache eviction."
+                    "Unable to read allocated memory, skipping memory-based cache eviction."
                 )
-                # If we've failed to read the current memory usage then we
-                # should stop trying to evict based on memory usage
-                evicting_due_to_memory = False
 
-        # If we do lots of work at once we yield to allow other stuff to happen.
-        if (i + 1) % 10000 == 0:
-            logger.debug("Waiting during drop")
-            if node.last_access_ts_secs > now - expiry_seconds:
-                await clock.sleep(0.5)
-            else:
-                await clock.sleep(0)
-            logger.debug("Waking during drop")
+        while node is not GLOBAL_ROOT:
+            # Only the root node isn't a `_TimedListNode`.
+            assert isinstance(node, _TimedListNode)
 
-        node = next_node
+            # if node has not aged past expiry_seconds and we are not evicting due to memory usage, there's
+            # nothing to do here
+            if (
+                node.last_access_ts_secs > now - expiry_seconds
+                and not evicting_due_to_memory
+            ):
+                break
 
-        # If we've yielded then our current node may have been evicted, so we
-        # need to check that its still valid.
-        if node.prev_node is None:
-            break
+            # if entry is newer than min_cache_entry_ttl then do not evict and don't evict anything newer
+            if (
+                evicting_due_to_memory
+                and now - node.last_access_ts_secs < min_cache_ttl
+            ):
+                break
 
-        i += 1
+            cache_entry = node.get_cache_entry()
+            next_node = node.prev_node
 
-    logger.info("Dropped %d items from caches", i)
+            # The node should always have a reference to a cache entry and a valid
+            # `prev_node`, as we only drop them when we remove the node from the
+            # list.
+            assert next_node is not None
+            assert cache_entry is not None
+            cache_entry.drop_from_cache()
+
+            # Check mem allocation periodically if we are evicting a bunch of caches
+            if jemalloc_interface and evicting_due_to_memory and (i + 1) % 100 == 0:
+                try:
+                    jemalloc_interface.refresh_stats()
+                    mem_usage = jemalloc_interface.get_stat("allocated")
+                    if mem_usage < target_cache_memory_usage:
+                        evicting_due_to_memory = False
+                        logger.info("Stop memory-based cache eviction.")
+                except Exception:
+                    logger.warning(
+                        "Unable to read allocated memory, this may affect memory-based cache eviction."
+                    )
+                    # If we've failed to read the current memory usage then we
+                    # should stop trying to evict based on memory usage
+                    evicting_due_to_memory = False
+
+            # If we do lots of work at once we yield to allow other stuff to happen.
+            if (i + 1) % 10000 == 0:
+                logger.debug("Waiting during drop")
+                if node.last_access_ts_secs > now - expiry_seconds:
+                    await clock.sleep(0.5)
+                else:
+                    await clock.sleep(0)
+                logger.debug("Waking during drop")
+
+            node = next_node
+
+            # If we've yielded then our current node may have been evicted, so we
+            # need to check that its still valid.
+            if node.prev_node is None:
+                break
+
+            i += 1
+
+        logger.info("Dropped %d items from caches", i)
+
+    return run_as_background_process(
+        "LruCache._expire_old_entries",
+        server_name,
+        _internal_expire_old_entries,
+        clock,
+        expiry_seconds,
+        autotune_config,
+    )
 
 
 def setup_expire_lru_cache_entries(hs: "HomeServer") -> None:
@@ -234,10 +255,12 @@ def setup_expire_lru_cache_entries(hs: "HomeServer") -> None:
     global USE_GLOBAL_LIST
     USE_GLOBAL_LIST = True
 
+    server_name = hs.hostname
     clock = hs.get_clock()
     clock.looping_call(
         _expire_old_entries,
         30 * 1000,
+        server_name,
         clock,
         expiry_time,
         hs.config.caches.cache_autotuning,
@@ -376,9 +399,43 @@ class LruCache(Generic[KT, VT]):
     If cache_type=TreeCache, all keys must be tuples.
     """
 
+    @overload
     def __init__(
         self,
+        *,
         max_size: int,
+        server_name: str,
+        cache_name: str,
+        cache_type: Type[Union[dict, TreeCache]] = dict,
+        size_callback: Optional[Callable[[VT], int]] = None,
+        metrics_collection_callback: Optional[Callable[[], None]] = None,
+        apply_cache_factor_from_config: bool = True,
+        clock: Optional[Clock] = None,
+        prune_unread_entries: bool = True,
+        extra_index_cb: Optional[Callable[[KT, VT], KT]] = None,
+    ): ...
+
+    @overload
+    def __init__(
+        self,
+        *,
+        max_size: int,
+        server_name: Literal[None] = None,
+        cache_name: Literal[None] = None,
+        cache_type: Type[Union[dict, TreeCache]] = dict,
+        size_callback: Optional[Callable[[VT], int]] = None,
+        metrics_collection_callback: Optional[Callable[[], None]] = None,
+        apply_cache_factor_from_config: bool = True,
+        clock: Optional[Clock] = None,
+        prune_unread_entries: bool = True,
+        extra_index_cb: Optional[Callable[[KT, VT], KT]] = None,
+    ): ...
+
+    def __init__(
+        self,
+        *,
+        max_size: int,
+        server_name: Optional[str] = None,
         cache_name: Optional[str] = None,
         cache_type: Type[Union[dict, TreeCache]] = dict,
         size_callback: Optional[Callable[[VT], int]] = None,
@@ -386,13 +443,19 @@ class LruCache(Generic[KT, VT]):
         apply_cache_factor_from_config: bool = True,
         clock: Optional[Clock] = None,
         prune_unread_entries: bool = True,
+        extra_index_cb: Optional[Callable[[KT, VT], KT]] = None,
     ):
         """
         Args:
             max_size: The maximum amount of entries the cache can hold
 
-            cache_name: The name of this cache, for the prometheus metrics. If unset,
-                no metrics will be reported on this cache.
+            server_name: The homeserver name that this cache is associated with
+                (used to label the metric) (`hs.hostname`). Must be set if `cache_name` is
+                set. If unset, no metrics will be reported on this cache.
+
+            cache_name: The name of this cache, for the prometheus metrics. Must be set
+                if `server_name` is set. If unset, no metrics will be reported on this
+                cache.
 
             cache_type:
                 type of underlying cache to be used. Typically one of dict
@@ -416,11 +479,25 @@ class LruCache(Generic[KT, VT]):
             prune_unread_entries: If True, cache entries that haven't been read recently
                 will be evicted from the cache in the background. Set to False to
                 opt-out of this behaviour.
+
+            extra_index_cb: If provided, the cache keeps a second index from a
+                (different) key to a cache entry based on the return value of
+                the callback. This can then be used to invalidate entries based
+                on the second type of key.
+
+                For example, for the event cache this would be a callback that
+                maps an event to its room ID, allowing invalidation of all
+                events in a given room.
+
+                Note: Though the two types of key have the same type, they are
+                in different namespaces.
+
+                Note: The new key does not have to be unique.
         """
         # Default `clock` to something sensible. Note that we rename it to
         # `real_clock` so that mypy doesn't think its still `Optional`.
         if clock is None:
-            real_clock = Clock(cast(IReactorTime, reactor))
+            real_clock = Clock(cast(ISynapseThreadlessReactor, reactor))
         else:
             real_clock = clock
 
@@ -442,11 +519,12 @@ class LruCache(Generic[KT, VT]):
         # do yet when we get resized.
         self._on_resize: Optional[Callable[[], None]] = None
 
-        if cache_name is not None:
+        if cache_name is not None and server_name is not None:
             metrics: Optional[CacheMetric] = register_cache(
-                "lru_cache",
-                cache_name,
-                self,
+                cache_type="lru_cache",
+                cache_name=cache_name,
+                cache=self,
+                server_name=server_name,
                 collect_callback=metrics_collection_callback,
             )
         else:
@@ -463,6 +541,8 @@ class LruCache(Generic[KT, VT]):
 
         lock = threading.Lock()
 
+        extra_index: Dict[KT, Set[KT]] = {}
+
         def evict() -> None:
             while cache_len() > self.max_size:
                 # Get the last node in the list (i.e. the oldest node).
@@ -521,6 +601,11 @@ class LruCache(Generic[KT, VT]):
             if size_callback:
                 cached_cache_len[0] += size_callback(node.value)
 
+            if extra_index_cb:
+                index_key = extra_index_cb(node.key, node.value)
+                mapped_keys = extra_index.setdefault(index_key, set())
+                mapped_keys.add(node.key)
+
             if caches.TRACK_MEMORY_USAGE and metrics:
                 metrics.inc_memory_usage(node.memory)
 
@@ -537,6 +622,14 @@ class LruCache(Generic[KT, VT]):
 
             node.run_and_clear_callbacks()
 
+            if extra_index_cb:
+                index_key = extra_index_cb(node.key, node.value)
+                mapped_keys = extra_index.get(index_key)
+                if mapped_keys is not None:
+                    mapped_keys.discard(node.key)
+                    if not mapped_keys:
+                        extra_index.pop(index_key, None)
+
             if caches.TRACK_MEMORY_USAGE and metrics:
                 metrics.dec_memory_usage(node.memory)
 
@@ -549,8 +642,7 @@ class LruCache(Generic[KT, VT]):
             callbacks: Collection[Callable[[], None]] = ...,
             update_metrics: bool = ...,
             update_last_access: bool = ...,
-        ) -> Optional[VT]:
-            ...
+        ) -> Optional[VT]: ...
 
         @overload
         def cache_get(
@@ -559,8 +651,7 @@ class LruCache(Generic[KT, VT]):
             callbacks: Collection[Callable[[], None]] = ...,
             update_metrics: bool = ...,
             update_last_access: bool = ...,
-        ) -> Union[T, VT]:
-            ...
+        ) -> Union[T, VT]: ...
 
         @synchronized
         def cache_get(
@@ -603,16 +694,14 @@ class LruCache(Generic[KT, VT]):
             key: tuple,
             default: Literal[None] = None,
             update_metrics: bool = True,
-        ) -> Union[None, Iterable[Tuple[KT, VT]]]:
-            ...
+        ) -> Union[None, Iterable[Tuple[KT, VT]]]: ...
 
         @overload
         def cache_get_multi(
             key: tuple,
             default: T,
             update_metrics: bool = True,
-        ) -> Union[T, Iterable[Tuple[KT, VT]]]:
-            ...
+        ) -> Union[T, Iterable[Tuple[KT, VT]]]: ...
 
         @synchronized
         def cache_get_multi(
@@ -697,12 +786,10 @@ class LruCache(Generic[KT, VT]):
                 return value
 
         @overload
-        def cache_pop(key: KT, default: Literal[None] = None) -> Optional[VT]:
-            ...
+        def cache_pop(key: KT, default: Literal[None] = None) -> Optional[VT]: ...
 
         @overload
-        def cache_pop(key: KT, default: T) -> Union[T, VT]:
-            ...
+        def cache_pop(key: KT, default: T) -> Union[T, VT]: ...
 
         @synchronized
         def cache_pop(key: KT, default: Optional[T] = None) -> Union[None, T, VT]:
@@ -748,6 +835,8 @@ class LruCache(Generic[KT, VT]):
             if size_callback:
                 cached_cache_len[0] = 0
 
+            extra_index.clear()
+
             if caches.TRACK_MEMORY_USAGE and metrics:
                 metrics.clear_memory_usage()
 
@@ -755,6 +844,28 @@ class LruCache(Generic[KT, VT]):
         def cache_contains(key: KT) -> bool:
             return key in cache
 
+        @synchronized
+        def cache_invalidate_on_extra_index(index_key: KT) -> None:
+            """Invalidates all entries that match the given extra index key.
+
+            This can only be called when `extra_index_cb` was specified.
+            """
+
+            assert extra_index_cb is not None
+
+            keys = extra_index.pop(index_key, None)
+            if not keys:
+                return
+
+            for key in keys:
+                node = cache.pop(key, None)
+                if not node:
+                    continue
+
+                evicted_len = delete_node(node)
+                if metrics:
+                    metrics.inc_evictions(EvictionReason.invalidation, evicted_len)
+
         # make sure that we clear out any excess entries after we get resized.
         self._on_resize = evict
 
@@ -771,6 +882,7 @@ class LruCache(Generic[KT, VT]):
         self.len = synchronized(cache_len)
         self.contains = cache_contains
         self.clear = cache_clear
+        self.invalidate_on_extra_index = cache_invalidate_on_extra_index
 
     def __getitem__(self, key: KT) -> VT:
         result = self.get(key, _Sentinel.sentinel)
@@ -864,6 +976,9 @@ class AsyncLruCache(Generic[KT, VT]):
         # This method should invalidate any external cache and then invalidate the LruCache.
         return self._lru_cache.invalidate(key)
 
+    def invalidate_on_extra_index_local(self, index_key: KT) -> None:
+        self._lru_cache.invalidate_on_extra_index(index_key)
+
     def invalidate_local(self, key: KT) -> None:
         """Remove an entry from the local cache
 
