@@ -21,12 +21,12 @@
 from typing import Any, List, Mapping, Optional, Sequence, Union
 from unittest.mock import Mock
 
-from twisted.test.proto_helpers import MemoryReactor
+from twisted.internet.testing import MemoryReactor
 
 from synapse.appservice import ApplicationService
 from synapse.server import HomeServer
-from synapse.types import JsonDict
-from synapse.util import Clock
+from synapse.types import JsonDict, UserID
+from synapse.util.clock import Clock
 
 from tests import unittest
 from tests.unittest import override_config
@@ -41,7 +41,7 @@ class ApplicationServiceApiTestCase(unittest.HomeserverTestCase):
         self.api = hs.get_application_service_api()
         self.service = ApplicationService(
             id="unique_identifier",
-            sender="@as:test",
+            sender=UserID.from_string("@as:test"),
             url=URL,
             token="unused",
             hs_token=TOKEN,
