@@ -26,7 +26,7 @@ from synapse.handlers.presence import PresenceHandler
 from synapse.rest.client import presence
 from synapse.server import HomeServer
 from synapse.types import UserID
-from synapse.util import Clock
+from synapse.util.clock import Clock
 
 from tests import unittest
 from tests.unittest import override_config
