@@ -43,7 +43,7 @@ from synapse.events.snapshot import EventContext
 from synapse.state import StateHandler, StateResolutionHandler, _make_state_cache_entry
 from synapse.types import MutableStateMap, StateMap
 from synapse.types.state import StateFilter
-from synapse.util import Clock
+from synapse.util.clock import Clock
 from synapse.util.macaroons import MacaroonGenerator
 
 from tests import unittest
@@ -149,7 +149,7 @@ class _DummyStore:
     async def get_partial_state_events(
         self, event_ids: Collection[str]
     ) -> Dict[str, bool]:
-        return {e: False for e in event_ids}
+        return dict.fromkeys(event_ids, False)
 
     async def get_state_group_delta(
         self, name: str
