@@ -21,8 +21,8 @@
 
 from typing import List, cast
 
-from synapse.util import Clock
 from synapse.util.caches.expiringcache import ExpiringCache
+from synapse.util.clock import Clock
 
 from tests.utils import MockClock
 
@@ -33,7 +33,10 @@ class ExpiringCacheTestCase(unittest.HomeserverTestCase):
     def test_get_set(self) -> None:
         clock = MockClock()
         cache: ExpiringCache[str, str] = ExpiringCache(
-            "test", cast(Clock, clock), max_len=1
+            cache_name="test",
+            server_name="testserver",
+            clock=cast(Clock, clock),
+            max_len=1,
         )
 
         cache["key"] = "value"
@@ -43,7 +46,10 @@ class ExpiringCacheTestCase(unittest.HomeserverTestCase):
     def test_eviction(self) -> None:
         clock = MockClock()
         cache: ExpiringCache[str, str] = ExpiringCache(
-            "test", cast(Clock, clock), max_len=2
+            cache_name="test",
+            server_name="testserver",
+            clock=cast(Clock, clock),
+            max_len=2,
         )
 
         cache["key"] = "value"
@@ -59,7 +65,11 @@ class ExpiringCacheTestCase(unittest.HomeserverTestCase):
     def test_iterable_eviction(self) -> None:
         clock = MockClock()
         cache: ExpiringCache[str, List[int]] = ExpiringCache(
-            "test", cast(Clock, clock), max_len=5, iterable=True
+            cache_name="test",
+            server_name="testserver",
+            clock=cast(Clock, clock),
+            max_len=5,
+            iterable=True,
         )
 
         cache["key"] = [1]
@@ -79,7 +89,10 @@ class ExpiringCacheTestCase(unittest.HomeserverTestCase):
     def test_time_eviction(self) -> None:
         clock = MockClock()
         cache: ExpiringCache[str, int] = ExpiringCache(
-            "test", cast(Clock, clock), expiry_ms=1000
+            cache_name="test",
+            server_name="testserver",
+            clock=cast(Clock, clock),
+            expiry_ms=1000,
         )
 
         cache["key"] = 1
