@@ -15,7 +15,12 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         Providing a prefilled cache to StreamChangeCache will result in a cache
         with the prefilled-cache entered in.
         """
-        cache = StreamChangeCache("#test", 1, prefilled_cache={"user@foo.com": 2})
+        cache = StreamChangeCache(
+            name="#test",
+            server_name=self.hs.hostname,
+            current_stream_pos=1,
+            prefilled_cache={"user@foo.com": 2},
+        )
         self.assertTrue(cache.has_entity_changed("user@foo.com", 1))
 
     def test_has_entity_changed(self) -> None:
@@ -23,7 +28,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         StreamChangeCache.entity_has_changed will mark entities as changed, and
         has_entity_changed will observe the changed entities.
         """
-        cache = StreamChangeCache("#test", 3)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=3
+        )
 
         cache.entity_has_changed("user@foo.com", 6)
         cache.entity_has_changed("bar@baz.net", 7)
@@ -61,7 +68,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         StreamChangeCache.entity_has_changed will respect the max size and
         purge the oldest items upon reaching that max size.
         """
-        cache = StreamChangeCache("#test", 1, max_size=2)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=1, max_size=2
+        )
 
         cache.entity_has_changed("user@foo.com", 2)
         cache.entity_has_changed("bar@baz.net", 3)
@@ -100,7 +109,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         entities since the given position.  If the position is before the start
         of the known stream, it returns None instead.
         """
-        cache = StreamChangeCache("#test", 1)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=1
+        )
 
         cache.entity_has_changed("user@foo.com", 2)
         cache.entity_has_changed("bar@baz.net", 3)
@@ -148,7 +159,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         stream position is before it, it will return True, otherwise False if
         the cache has no entries.
         """
-        cache = StreamChangeCache("#test", 1)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=1
+        )
 
         # With no entities, it returns True for the past, present, and False for
         # the future.
@@ -175,7 +188,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         stream position is earlier than the earliest known position, it will
         return all of the entities queried for.
         """
-        cache = StreamChangeCache("#test", 1)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=1
+        )
 
         cache.entity_has_changed("user@foo.com", 2)
         cache.entity_has_changed("bar@baz.net", 3)
@@ -242,7 +257,9 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
         recent point where the entity could have changed.  If the entity is not
         known, the stream start is provided instead.
         """
-        cache = StreamChangeCache("#test", 1)
+        cache = StreamChangeCache(
+            name="#test", server_name=self.hs.hostname, current_stream_pos=1
+        )
 
         cache.entity_has_changed("user@foo.com", 2)
         cache.entity_has_changed("bar@baz.net", 3)
@@ -255,3 +272,33 @@ class StreamChangeCacheTests(unittest.HomeserverTestCase):
 
         # Unknown entities will return None
         self.assertEqual(cache.get_max_pos_of_last_change("not@here.website"), None)
+
+    def test_all_entities_changed(self) -> None:
+        """
+        `StreamChangeCache.all_entities_changed(...)` will mark all entites as changed.
+        """
+        cache = StreamChangeCache(
+            name="#test",
+            server_name=self.hs.hostname,
+            current_stream_pos=1,
+            max_size=10,
+        )
+
+        cache.entity_has_changed("user@foo.com", 2)
+        cache.entity_has_changed("bar@baz.net", 3)
+        cache.entity_has_changed("user@elsewhere.org", 4)
+
+        cache.all_entities_changed(5)
+
+        # Everything should be marked as changed before the stream position where the
+        # change occurred.
+        self.assertTrue(cache.has_entity_changed("user@foo.com", 4))
+        self.assertTrue(cache.has_entity_changed("bar@baz.net", 4))
+        self.assertTrue(cache.has_entity_changed("user@elsewhere.org", 4))
+
+        # Nothing should be marked as changed at/after the stream position where the
+        # change occurred. In other words, nothing has changed since the stream position
+        # 5.
+        self.assertFalse(cache.has_entity_changed("user@foo.com", 5))
+        self.assertFalse(cache.has_entity_changed("bar@baz.net", 5))
+        self.assertFalse(cache.has_entity_changed("user@elsewhere.org", 5))
